//////////////////////////////////////////////////////////////
//                                                          //
//             MODULE DE STRUCTURES DE DONNEES              //
//                                                          //
//        MODULE D'ANIMATION DE LA DEMO D'ARBRES AVL        //
//                                                          //
// Ce module assure les fonctions generales d'affichage et  //
//    de surveillance de l'execution de l'animation des     //
//    operations sur les arbres AVL. Le controle se fait au //
//    moyen de boutons et d'un ascenceur. Il est possible   //
//    d'arreter, suspendre, reprendre ou executer en pas a  //
//    pas l'animation. L'ascenseur controle la vitesse      //
//    d'execution de lanimation. Le chemin courant est     //
//    indique avec un fond vert. Le noeud courant est       //
//    indique par un fond magenta. Les desequilibres sont   //
//    indiques par un (ou deux) trait horizontal rouge sous //
//    le noeud correspondant du cote du desequilibre.       //
//                                                          //
// Auteur: Christian CARREZ,                                //
// Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   //
// Derniere modification: 17 fevrier 2000                   //
//////////////////////////////////////////////////////////////

package Demonstration_AVL;

import java.awt.*; 
import java.awt.event.*;
import Commun_Demonstration.* ;

class Position_Affichage {
  Noeud_AVL Le_Noeud;
  boolean A_Gauche; 
  int Pas_H;
  int cx;
  int cy;
  public Position_Affichage () {} 
} 

class NatureOperation extends Panel implements ActionListener { 
  private TextField Texte_Valeur;
  private TextField Phase; 
  int Valeur_Selectionnee;
  int Position; 
  final int RIEN = 0; 
  final int POSITIONNER = 1; 
  final int AJOUTER = 2; 
  final int SUPPRIMER = 3; 
  int Operation = RIEN;

  public NatureOperation () { 
    Button b = null ; 
    Phase = new TextField ("Attente", 30);
    Phase.setEditable (false);
//    Phase.setFont (new Font ("middleFont", 0, Taille_Fonte));
    add(Phase); 
    add (new Label ("Valeur: "));
    Texte_Valeur = new TextField ("", 2); 
    add(Texte_Valeur);
    b = new Button ("Positionner");
    add(b) ; 
    b.addActionListener (this) ; 
    b = new Button ("Ajouter");
    add(b) ; 
    b.addActionListener (this) ; 
    b = new Button ("Supprimer");
    add(b) ; 
    b.addActionListener (this) ; 
  }
 
  public void actionPerformed (ActionEvent evt) { 
    String Etiquette = evt.getActionCommand();
    if (Operation != RIEN) return;
    if (Etiquette.equals ("Positionner")) { 
      try {
        Valeur_Selectionnee =Integer.parseInt(Texte_Valeur.getText());
        Operation = POSITIONNER; 
      } catch (Exception e) { } 
    } else if (Etiquette.equals ("Ajouter")){ 
      try {
        Valeur_Selectionnee =Integer.parseInt(Texte_Valeur.getText());
        Operation = AJOUTER; 
      } catch (Exception e) { }
    } else if (Etiquette.equals ("Supprimer")) { 
        Operation = SUPPRIMER; 
    } 
  }

  public void Affiche_Phase (String S) { 
    Phase.setText (S);
  } 
} 

class Affiche_AVL extends Canvas { 

  private Arbre_AVL Arbre;
  private Animation_AVL Demo;
  private Image Image_Locale; 
  private Position_Affichage Chemin[];
  private int Sommet = -1;
  int Pas_V;

  private int Demi_H_Label; // par rapport au centre
  private int Demi_V_Label;
  private int Demi_H_Noeud; // taille du noeud
  private int Demi_V_Noeud; 
  private int Taille_Fonte;
 
  public Affiche_AVL (Animation_AVL Demo, 
                       int TF_Anim, Arbre_AVL L_Arbre) { 
    this.Demo = Demo; 
    Taille_Fonte = TF_Anim;
    Demi_V_Label = (TF_Anim / 2) - 1;
    Demi_H_Label = Demi_V_Label + 2;
    Demi_V_Noeud = Demi_H_Label;
    Demi_H_Noeud = Demi_V_Noeud + 1; 
    Arbre = L_Arbre;
    Chemin = new Position_Affichage [10]; 
    for (int i = 0; i < Chemin.length; i++) 
      Chemin[i] = new Position_Affichage ();
    Image_Locale = createImage (1000, 1000) ; 
  }

  private void Dessiner_Un_Noeud_Vide_Eventuel (Graphics g) { 
    Position_Affichage R = Chemin[Sommet]; 
    Position_Affichage Q; 
    if (Sommet != Arbre.Sommet) return;
    if (Sommet > 0) { 
      Q = Chemin[Sommet-1];
      if ((Q.Le_Noeud != Arbre.Chemin[Sommet-1].Le_Noeud)
         ||(Q.A_Gauche != Arbre.Chemin[Sommet-1].A_Gauche))
        return;
      g.setColor (Color.black); 
      if (Q.A_Gauche) {
        g.drawLine (Q.cx - Demi_H_Noeud/2, Q.cy + Demi_V_Noeud,
                    R.cx, R.cy - Demi_V_Noeud); 
      } else { 
        g.drawLine (Q.cx + Demi_H_Noeud/2, Q.cy + Demi_V_Noeud,
                    R.cx, R.cy - Demi_V_Noeud); 
      }
    } 
    g.setColor (Color.green);
    g.drawRect (R.cx - Demi_H_Noeud, R.cy - Demi_V_Noeud,
                2 * Demi_H_Noeud, 2 * Demi_V_Noeud);
    g.setColor (Color.black); 
  }
  private void Dessiner_Le_Noeud (Graphics g) { 
    Position_Affichage R = Chemin[Sommet]; 
    Position_Affichage Q; 
    if (Sommet > 0) { 
      g.setColor (Color.black);
      Q = Chemin[Sommet-1];
      if (Q.A_Gauche) {
        g.drawLine (Q.cx - Demi_H_Noeud/2, Q.cy + Demi_V_Noeud,
                    R.cx, R.cy - Demi_V_Noeud); 
        g.setColor (Color.red);
        if (Q.Le_Noeud.Deseq > 0) 
          g.drawLine (Q.cx - Demi_H_Noeud, Q.cy + Demi_V_Noeud + 1,
                      Q.cx, Q.cy + Demi_V_Noeud + 1); 
        if (Q.Le_Noeud.Deseq == 2) 
          g.drawLine (Q.cx - Demi_H_Noeud, Q.cy + Demi_V_Noeud+4,
                      Q.cx, Q.cy + Demi_V_Noeud+4);
      } else { 
        g.drawLine (Q.cx + Demi_H_Noeud/2, Q.cy + Demi_V_Noeud,
                    R.cx, R.cy - Demi_V_Noeud); 
        g.setColor (Color.red);
        if (Q.Le_Noeud.Deseq < 0) 
          g.drawLine (Q.cx, Q.cy + Demi_V_Noeud + 1,
                      Q.cx + Demi_H_Noeud, Q.cy + Demi_V_Noeud + 1); 
        if (Q.Le_Noeud.Deseq == -2) 
          g.drawLine (Q.cx, Q.cy + Demi_V_Noeud + 4,
                      Q.cx + Demi_H_Noeud, Q.cy + Demi_V_Noeud + 4);
      }
    } 
    if (R.Le_Noeud == Demo.Noeud_Courant) 
      g.setColor (Color.magenta);
    else if ((Sommet <= Arbre.Sommet)
    && (R.Le_Noeud == Arbre.Chemin[Sommet].Le_Noeud))
      g.setColor (Color.green);
    else 
      g.setColor (Color.pink);
    g.fillRect (R.cx - Demi_H_Noeud, R.cy - Demi_V_Noeud,
                2 * Demi_H_Noeud, 2 * Demi_V_Noeud);
    g.setColor (Color.black);
    g.drawString("" + R.Le_Noeud.Contenu, 
                 R.cx - Demi_H_Label, R.cy + Demi_V_Label);
  }
  private void Descendre (boolean A_Gauche) { 
    Position_Affichage R = Chemin[Sommet]; 
    R.A_Gauche = A_Gauche;
    Sommet = Sommet + 1; 
    Chemin[Sommet].Le_Noeud = (A_Gauche ? R.Le_Noeud.Gauche 
                                        : R.Le_Noeud.Droite);
    Chemin[Sommet].cy = R.cy + Pas_V;
    Chemin[Sommet].Pas_H = R.Pas_H / 2;
    Chemin[Sommet].cx = R.cx - Chemin[Sommet].Pas_H;
    if (!A_Gauche) Chemin[Sommet].cx = Chemin[Sommet].cx + R.Pas_H;
  }
  private void Extremite_Bord_Gauche (Graphics g) { 
    while (Chemin[Sommet].Le_Noeud != null) { 
      Dessiner_Le_Noeud (g);
      Descendre (true); 
    } 
    Dessiner_Un_Noeud_Vide_Eventuel (g);
    Sommet = Sommet - 1;
  } 
  private void Remonter_Bord_Droit () {
    while ((Sommet > 0) && !Chemin[Sommet - 1].A_Gauche) 
      Sommet = Sommet - 1; // remontee depuis la droite
    Sommet = Sommet - 1;
  } 
  private void Positionner_Sur_Premier(Graphics g) { 
    Sommet = 0;
    Extremite_Bord_Gauche (g); 
  } 
  private void Passer_Au_Suivant (Graphics g) {
    Descendre (false);
    if (Chemin[Sommet].Le_Noeud != null) { 
      Extremite_Bord_Gauche (g); 
    } else { 
      Dessiner_Un_Noeud_Vide_Eventuel (g); 
      Remonter_Bord_Droit ();
    }
  } 
  private boolean A_La_Fin ( ) {return Sommet == -1;} 

  public void paint (Graphics g) { 
    Rectangle r = getBounds() ;
    Pas_V = (r.height - 2 * Demi_V_Noeud - 20) / (Chemin.length-2); 
    Chemin[0].Le_Noeud = Arbre.Chemin[0].Le_Noeud;
    Chemin[0].Pas_H = (r.width - 20) / 2;
    Chemin[0].cx = Chemin[0].Pas_H;
    Chemin[0].cy = 10 + Demi_V_Noeud;
    g.setFont (new Font("bigFont", 0, Taille_Fonte));
    Positionner_Sur_Premier (g);
    while (!A_La_Fin ()) { 
      Passer_Au_Suivant (g);
    } 
  } 

  public void update (Graphics g) { 
    Rectangle r = getBounds() ;
    if (Image_Locale == null)
      Image_Locale = createImage (1000, 1000);
    Graphics GL = Image_Locale.getGraphics();
    GL.setColor (getBackground ());
    GL.fillRect (0, 0, r.width, r.height);
    paint (GL);
    g.drawImage (Image_Locale, 0, 0, this);
  }
}

public class Animation_AVL extends Thread implements Affichage { 
  private NatureOperation natureOperation ;
  private Affiche_AVL afficheDemo; 
  private ControleDemo controleDemo ;
  private Frame f;
  private Arbre_AVL A;

  Noeud_AVL Noeud_Courant;
  int ieme [];

  public Animation_AVL (int[] V_Init, 
                         int L_Anim, int H_Anim, int TF_Anim) { 
    f = new Frame ("Animation sur les arbres AVL");
    int Pas = 99 / V_Init.length;
    ieme = new int [V_Init.length];
    for (int i = 0; i < ieme.length; i++) { 
      ieme[i] = V_Init[i] * Pas; 
    } 
    f.setLayout(new BorderLayout()) ;
    natureOperation = new NatureOperation () ; 
    f.add("North", natureOperation); 
    A = new Arbre_AVL (this);
    afficheDemo = new Affiche_AVL (this, TF_Anim, A); 
    f.add("Center", afficheDemo);
    controleDemo = new ControleDemo (true) ;
    f.add("South", controleDemo); 
    f.setSize (L_Anim, H_Anim); 
    f.show();
  } 

  public void Redessiner (Noeud_AVL Le_Noeud) throws StopAnim { 
    Noeud_Courant = Le_Noeud;
    afficheDemo.repaint ();
    controleDemo.Suspendre();
  } 

  public void L_Animation () throws StopAnim { 
    int K; 
    for (K = 0; K < ieme.length; K++) { 
      natureOperation.Affiche_Phase ("Ajout de la valeur "+ieme[K]); 
      try {
        A.Ajouter (ieme[K]); 
        natureOperation.Affiche_Phase ("Fin de lajout de "+ieme[K]);
      } catch (IllegalArgumentException e) {
        natureOperation.Affiche_Phase ("Deja existant ");
      }
      Redessiner (null);
      controleDemo.FinEtape();
    } 
    natureOperation.Affiche_Phase ("Plus de valeurs a ajouter ");
    while (!controleDemo.DemandeArret()) { 
      if (natureOperation.Operation == natureOperation.RIEN) try { 
          sleep (100);
      } catch (Exception e) {
      } 
      if (!controleDemo.DemandeArret() 
           && natureOperation.Operation != natureOperation.RIEN) {
        switch (natureOperation.Operation) { 
          case natureOperation.POSITIONNER: 
            natureOperation.Affiche_Phase ("Positionnement ");
            try { 
              A.Positionner (natureOperation.Valeur_Selectionnee); 
            } catch (IllegalArgumentException e) { 
              natureOperation.Affiche_Phase ("Non trouve ");
            } 
          break; 
          case natureOperation.AJOUTER: 
            natureOperation.Affiche_Phase ("Ajout de la valeur ");
            try {
              A.Ajouter (natureOperation.Valeur_Selectionnee); 
            } catch (IllegalArgumentException e) { 
              natureOperation.Affiche_Phase ("Deja existant ");
            } break; 
          case natureOperation.SUPPRIMER: 
            natureOperation.Affiche_Phase ("Supprimer le courant ");
            try {
              A.Supprimer_Courant ();
            } catch (IllegalArgumentException e) { 
              natureOperation.Affiche_Phase ("Noeud inexistant ");
            } break; 
        } 
        natureOperation.Operation = natureOperation.RIEN;
        Redessiner (null); 
        natureOperation.Affiche_Phase ("Operation terminee ");
      }
    }
  }

  public void run () { 
    try { 
      L_Animation ();
    } catch (StopAnim e) { 
    } 
    controleDemo.AttenteArret();
    f.dispose();
  }
} 
