//////////////////////////////////////////////////////////////
//                                                          //
//          MODULE DE DEMONSTRATION DES ARBRES AVL          //
//                                                          //
//               SPECIFICATION DES ARBRES AVL               //
//                                                          //
// Ce module reprend les operations des arbres AVL, en      //
//    demandant l'affichage de l'arbre a des moments        //
//    choisis.                                              //
//                                                          //
// Auteur: Christian CARREZ,                                //
// Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   //
// Derniere modification: 25 novembre 1999                  //
//////////////////////////////////////////////////////////////

package Demonstration_AVL;

import Commun_Demonstration.* ;

class Position {
  public Noeud_AVL Le_Noeud;
  public boolean A_Gauche;
  public Position (Noeud_AVL P, boolean Sens) { 
    Le_Noeud = P; A_Gauche = Sens; 
  } 
  public Position () {Le_Noeud = null; A_Gauche = true;} 
  public Noeud_AVL Descendre (Noeud_AVL P, boolean Sens) { 
    Le_Noeud = P; 
    A_Gauche = Sens; 
    return (A_Gauche ? P.Gauche : P.Droite); 
  } 
  public void Raccrocher (Noeud_AVL P) { 
    if (A_Gauche) Le_Noeud.Gauche = P; 
    else Le_Noeud.Droite = P; 
  } 
} 

public class Arbre_AVL { 
  Position Chemin[];
  int Sommet = -1;
  Affichage Activite;

  public Arbre_AVL (Affichage X) { 
    Chemin = new Position [10]; 
    Chemin[0] = new Position ();
    Activite = X;
  }
  protected void Descendre (boolean A_Gauche) throws StopAnim { 
    Position R = Chemin[Sommet]; 
    R.A_Gauche = A_Gauche;
    Sommet = Sommet + 1;
    if (Chemin[Sommet] == null) Chemin[Sommet] = new Position();
    Chemin[Sommet].Le_Noeud = (A_Gauche ? R.Le_Noeud.Gauche 
                                        : R.Le_Noeud.Droite);
    Activite.Redessiner (Chemin[Sommet].Le_Noeud);
  } 
  protected void Raccrocher (Noeud_AVL P, int H) { 
    Chemin[H].Le_Noeud = P;
    if (H > 0) Chemin[H-1].Raccrocher (P); 
  }
  protected void Raccrocher (Noeud_AVL P) {Raccrocher (P, Sommet);} 
  protected void Extremite_Bord_Gauche () throws StopAnim { 
    while (Chemin[Sommet].Le_Noeud != null) Descendre (true);
    Sommet = Sommet - 1;
  } 
  protected void Remonter_Bord_Droit () throws StopAnim {
    while ((Sommet > 0) && !Chemin[Sommet - 1].A_Gauche) { 
      Sommet = Sommet - 1; // remontee depuis la droite
      Activite.Redessiner (Chemin[Sommet].Le_Noeud);
    } 
    Sommet = Sommet - 1; 
    if (Sommet >= 0) Activite.Redessiner (Chemin[Sommet].Le_Noeud);
    else Activite.Redessiner (null);
  } 
  protected boolean Recherche_Locale (int Sur) throws StopAnim {
    Noeud_AVL Le_Noeud = Chemin[0].Le_Noeud;
    Sommet = 0; 
    Activite.Redessiner (Chemin[Sommet].Le_Noeud);
    while (Le_Noeud != null) { 
      if (Le_Noeud.Contenu == Sur) return true;
      Descendre (Sur < Le_Noeud.Contenu);
      Le_Noeud = Chemin[Sommet].Le_Noeud;
    }
    return false;
  } 
  public void Positionner (int Sur) throws StopAnim { 
    if (Recherche_Locale (Sur)) return;
    Sommet = -1;
    throw new IllegalArgumentException (); // non trouv 
  }
  Noeud_AVL Rotation_Droite(Noeud_AVL Le_Noeud) {
    Noeud_AVL Result = Le_Noeud.Gauche;
    Le_Noeud.Gauche = Result.Droite;
    Result.Droite = Le_Noeud;
    return Result;
  }
  Noeud_AVL Rotation_Gauche(Noeud_AVL Le_Noeud) {
    Noeud_AVL Result = Le_Noeud.Droite;
    Le_Noeud.Droite = Result.Gauche;
    Result.Gauche = Le_Noeud;
    return Result;
  }
  Noeud_AVL Reequilibrage (Noeud_AVL Le_Noeud) {
    int D;
    if (Le_Noeud.Deseq == 2) {
      if (Le_Noeud.Gauche.Deseq == -1) {
                                    // rotation gauche droite
        Le_Noeud.Gauche = Rotation_Gauche (Le_Noeud.Gauche);
        Le_Noeud = Rotation_Droite (Le_Noeud); 
        D = Le_Noeud.Deseq;
        Le_Noeud.Gauche.Deseq = (D < 0 ? 1 : 0);
        Le_Noeud.Droite.Deseq = (D > 0 ?-1 : 0);
        Le_Noeud.Deseq = 0;
      } else {                          // rotation  droite
        Le_Noeud = Rotation_Droite (Le_Noeud); 
        D = Le_Noeud.Deseq;
        Le_Noeud.Droite.Deseq = 1 - D;
        Le_Noeud.Deseq = D - 1;
      } 
    } else if (Le_Noeud.Deseq == -2) { 
      if (Le_Noeud.Droite.Deseq == 1) {
                                    // rotation droite gauche
        Le_Noeud.Droite = Rotation_Droite (Le_Noeud.Droite);
        Le_Noeud = Rotation_Gauche (Le_Noeud);
        D = Le_Noeud.Deseq;
        Le_Noeud.Gauche.Deseq = (D < 0 ? 1 : 0);
        Le_Noeud.Droite.Deseq = (D > 0 ?-1 : 0);
        Le_Noeud.Deseq = 0;
      } else {                            // rotation  gauche
        Le_Noeud = Rotation_Gauche (Le_Noeud); 
        D = Le_Noeud.Deseq;
        Le_Noeud.Gauche.Deseq = -1 - D;
        Le_Noeud.Deseq = D + 1;
      } 
    } 
    return Le_Noeud;
  } 
  void Ajuster_Chemin_En_Adjonction (int En) { 
    if (Chemin[En].Le_Noeud.Deseq == 2) {
      if (!Chemin[En + 1].A_Gauche) {        // rot. gauche droite
        if (Sommet == En + 2) 
          Sommet = Sommet - 1;               // 2 noeuds en moins
        else if (!Chemin[En + 2].A_Gauche) 
          Chemin[En + 1] = Chemin[En];      // un 
        Chemin[En] = Chemin[En + 2]; 
        Chemin[En + 2] = null; // unicite des positions du chemin
      } else {                                 // rotation droite
        Chemin [En] = Chemin [En + 1];
        Chemin [En + 1] = Chemin [En + 2];
      } 
    } else if (Chemin [En].Le_Noeud.Deseq==-2){ 
      if (Chemin [En + 1].A_Gauche) {        // rot. droite gauche 
        if (Sommet == En + 2) 
          Sommet = Sommet - 1;
        else if (Chemin [En + 2].A_Gauche) 
          Chemin [En + 1] = Chemin [En];
        Chemin [En] = Chemin [En + 2]; 
        Chemin [En + 2] = null; // unicite des positions du chemin
      } else {                               // rotation gauche
        Chemin [En] = Chemin [En + 1];
        Chemin [En + 1] = Chemin [En + 2];
      } 
    } else return;                 // pas dajustement
                    // on remonte les autres noeuds du chemin
    for (int I = En + 2; I <= Sommet - 1; I++) 
      Chemin [I] = Chemin [I + 1];
    Chemin[Sommet] = null; // unicite des positions du chemin
    Sommet = Sommet - 1; 
  } 
  public void Ajouter (int Val) throws StopAnim {
    Noeud_AVL Le_Noeud; 
    if (Recherche_Locale (Val)) 
      throw new IllegalArgumentException (); // dj existant
    Le_Noeud = new Noeud_AVL();
    Le_Noeud.Contenu = Val; 
    Raccrocher (Le_Noeud); // devient la fin du chemin
    Activite.Redessiner (Le_Noeud);
    for (int I = Sommet - 1; I >= 0; I--) { 
                                     // correction des dsquilibres 
      Le_Noeud = Chemin [I].Le_Noeud;
      if (Chemin [I].A_Gauche) 
        Le_Noeud.Deseq ++;
      else
        Le_Noeud.Deseq --;
      Activite.Redessiner (Le_Noeud);
      Ajuster_Chemin_En_Adjonction (I);
      Le_Noeud = Reequilibrage (Le_Noeud);
      Raccrocher (Le_Noeud, I); 
      Activite.Redessiner (Le_Noeud);
      if (Le_Noeud.Deseq == 0) 
        return;                               // mme hauteur
    } 
  } 
  void Ajuster_Chemin_En_Suppression (int En) { 
    Noeud_AVL Le_Noeud = Chemin [En].Le_Noeud;
    boolean A_Gauche;
    if (Le_Noeud.Deseq == 2) { 
      Le_Noeud = Le_Noeud.Gauche;
      if (Le_Noeud.Deseq == -1) 
        Le_Noeud = Le_Noeud.Droite;     // rot. gauche droite
      A_Gauche = false;                 // il est avant le supprim
    } else if (Le_Noeud.Deseq == -2) { 
      Le_Noeud = Le_Noeud.Droite;
      if (Le_Noeud.Deseq == 1) 
        Le_Noeud = Le_Noeud.Gauche;      // rot. droite gauche
      A_Gauche = true;                   // il est avant le supprim
    } else return;       // pas de changement du chemin
    if (En <= Sommet) {      // en suppression sur le chemin
      for (int I = Sommet + 1; I >= En + 1; I--) 
        Chemin [I] = Chemin [I - 1]; // un noeud  insrer
      Chemin [En] = new Position(Le_Noeud, A_Gauche); 
      Sommet = Sommet + 1;
    } 
  } 
  public void Supprimer_Courant () throws StopAnim {
    Noeud_AVL Le_Noeud; 
    int Corriger_En; 
    if (Sommet == -1) throw new IllegalArgumentException ();
    Le_Noeud = Chemin[Sommet].Le_Noeud; 
                                // noeud physiquement  supprimer
    if ((Le_Noeud.Gauche == null) || (Le_Noeud.Droite == null)) { 
      Corriger_En = Sommet - 1; // au plus un fils 
      if (Le_Noeud.Gauche == null) { 
        Raccrocher (Le_Noeud.Droite); 
        Activite.Redessiner (Le_Noeud.Droite);
      } else { 
        Raccrocher (Le_Noeud.Gauche); 
        Activite.Redessiner (Le_Noeud.Gauche);
      } 
                                 // positionnement sur suivant 
      if (Le_Noeud.Droite == null) Remonter_Bord_Droit();//ascendant
      else { 
        Extremite_Bord_Gauche ();                  // descendant
      }
    } else {  //deux fils, recherche bord gauche du sous-arbre 
                    // droit qui est aussi la valeur suivante
      int Sauv_S = Sommet; // endroit du chemin ou lon doit etre
      Descendre (false);
      Extremite_Bord_Gauche ();
      Le_Noeud = Chemin[Sommet].Le_Noeud;      // a supprimer
      Chemin[Sauv_S].Le_Noeud.Contenu = Le_Noeud.Contenu; 
      Raccrocher (Le_Noeud.Droite); 
      Activite.Redessiner (Le_Noeud.Droite);
      Corriger_En = Sommet - 1;
      Sommet = Sauv_S;
    } 
    for (int I = Corriger_En; I >= 0; I--) { 
                                 // correction des dsquilibres 
      Le_Noeud = Chemin[I].Le_Noeud;
      if (Chemin [I].A_Gauche) Le_Noeud.Deseq--;
      else Le_Noeud.Deseq ++;
      Activite.Redessiner (Le_Noeud);
      Ajuster_Chemin_En_Suppression (I);
      Le_Noeud = Reequilibrage (Le_Noeud);
      Raccrocher (Le_Noeud, I); 
      Activite.Redessiner (Le_Noeud);
      if (Le_Noeud.Deseq != 0) return;           // mme hauteur
    } 
  } 
} 
