//////////////////////////////////////////////////////////////
//                                                          //
//         DEFINITION DES NOEUDS D'ARBRES BALANCES          //
//                                                          //
//                                                          //
// Auteur: Christian CARREZ,                                //
// Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   //
// Derniere modification: 23 mai 2000                       //
//////////////////////////////////////////////////////////////

package Demonstration_BAL;

public class Noeud_Balance {
  private int Liste_Valeurs [];
  private Noeud_Balance Liste_Fils []; 
  private int Longueur;
  private boolean EstFeuille;
  public Noeud_Balance (int Degre_Max) { 
    Liste_Valeurs = new int [Degre_Max];
    Liste_Fils = new Noeud_Balance [Degre_Max + 1]; 
    Longueur = 0;
    EstFeuille = true;
  }
  public void Initialiser_Feuille (int Val) { 
    Longueur = 1; // vider contenu eventuel
    Liste_Valeurs[0] = Val;    // y mettre la valeur
    EstFeuille = true; 
  } 

  public void Initialiser_Noeud (int Val_Separ, 
                                  Noeud_Balance Gauche, 
                                  Noeud_Balance Droite) {
    Longueur = 1; // vider contenu eventuel
    Liste_Valeurs[0] = Val_Separ;    // y mettre la valeur
    EstFeuille = false; 
    Liste_Fils[0] = Gauche; // mettre les 
    Liste_Fils[1] = Droite; // deux fils
  }
  public int Degre () { return Longueur ; } 
  public boolean Est_Feuille () { return EstFeuille; } 
  public int Rang (int La_Cle) { 
    int Debut = 0;                    // recherche de sa place
    int Fin = Longueur - 1;
    int Milieu;
    if (Fin < 0 || Liste_Valeurs[Fin] < La_Cle)
      return Fin + 2;
    while (Debut < Fin) { 
      Milieu = (Debut + Fin) / 2; 
      if (Liste_Valeurs[Milieu] < La_Cle ) Debut = Milieu + 1;
      else Fin = Milieu;
    } 
    return Debut + 1; 
  } 
  public int Ieme_Valeur (int Indice) { 
    return Liste_Valeurs[Indice - 1]; 
  } 
  public Noeud_Balance Ieme_Fils (int Indice) { 
    return Liste_Fils[Indice - 1]; 
  } 
  public void Inserer(int Indice, int Val) {
    for (int j = Longueur; j >= Indice; j--)           // deplacements
      Liste_Valeurs [j] = Liste_Valeurs [j - 1];
    Longueur = Longueur + 1;
    Liste_Valeurs [Indice - 1] = Val; 
  } 
  public void Inserer(int Indice, int Val, Noeud_Balance Q) {
    for (int j = Longueur; j >= Indice; j--)           // deplacements
      Liste_Valeurs [j] = Liste_Valeurs [j - 1];
    Longueur = Longueur + 1;
    Liste_Valeurs [Indice - 1] = Val; 
    for (int j = Longueur; j >= Indice + 1; j--)       // deplacements
      Liste_Fils [j] = Liste_Fils [j - 1]; 
    Liste_Fils [Indice] = Q; 
  } 
  public void Supprimer (int Indice) {
    Longueur = Longueur - 1;
    for (int j = Indice; j <= Longueur; j++)           // deplacements
      Liste_Valeurs [j - 1] = Liste_Valeurs [j]; 
    if (!EstFeuille) { 
      for (int j = Indice + 1; j <= Longueur + 1; j++) // deplacements
      Liste_Fils [j - 1] = Liste_Fils [j]; 
    } 
  } 
  public void Changer_Valeur (int Indice, int Val) { 
    Liste_Valeurs[Indice - 1] = Val; 
   }
  public int Eclater(int En, Noeud_Balance Nouveau) { 
    int Val_Separ; 
    int K; 
    Val_Separ = Liste_Valeurs[En]; 
    Nouveau.Longueur = Longueur - En - 1;
    for (K = En + 2; K <= Longueur; K++) 
      Nouveau.Liste_Valeurs[K - En - 2] = Liste_Valeurs[K - 1]; 
    if (EstFeuille) Nouveau.EstFeuille = true;
    else { 
      Nouveau.EstFeuille = false;
      for (K = En + 2; K <= Longueur + 1; K++) 
        Nouveau.Liste_Fils[K - En - 2] = Liste_Fils[K - 1]; 
    } 
    Longueur = En;
    return Val_Separ; 
  } 
  public void Regrouper (int Indice, 
                          Noeud_Balance Gauche, 
                          Noeud_Balance Droite) {
    int Degre_G = Gauche.Degre ();
    int Degre_D = Droite.Degre ();
    int K; 
    Gauche.Liste_Valeurs[Degre_G] = Liste_Valeurs[Indice-1];
    for (K = 1; K <= Degre_D; K++) 
      Gauche.Liste_Valeurs[Degre_G+K] = Droite.Liste_Valeurs[K-1];
    if (!Gauche.Est_Feuille()) {
      for (K = 1; K <= Degre_D + 1; K++) 
        Gauche.Liste_Fils[Degre_G+K] = Droite.Liste_Fils[K-1];
      } 
    Gauche.Longueur = Gauche.Longueur + Droite.Longueur + 1;
    Supprimer (Indice); 
  } 
  public void Transfert_Droite_Gauche (int Indice,
                  Noeud_Balance Gauche, Noeud_Balance Droite) { 
    Gauche.Inserer (Gauche.Longueur + 1, Liste_Valeurs[Indice - 1],
                                         Droite.Liste_Fils[0]); 
    Liste_Valeurs[Indice - 1] = Droite.Liste_Valeurs[0]; 
    Droite.Longueur = Droite.Longueur - 1;
    for (int j = 1; j <= Droite.Longueur; j++)        // deplacements
      Droite.Liste_Valeurs [j - 1] = Droite.Liste_Valeurs [j]; 
    if (!Droite.EstFeuille) { 
      for (int j = 1; j <= Droite.Longueur + 1; j++)  // deplacements
        Droite.Liste_Fils [j - 1] = Droite.Liste_Fils [j]; 
    }
  } 
  public void Transfert_Gauche_Droite (int Indice,
                  Noeud_Balance Gauche, Noeud_Balance Droite) {
    int Degre_G = Gauche.Degre ();
    for (int j = Droite.Longueur; j >= 1; j--)        // deplacements
      Droite.Liste_Valeurs [j] = Droite.Liste_Valeurs [j - 1];
    Droite.Longueur = Droite.Longueur + 1;
    Droite.Liste_Valeurs [0] = Liste_Valeurs[Indice - 1]; 
    Liste_Valeurs[Indice - 1] = Gauche.Liste_Valeurs[Degre_G - 1];
    if (!Droite.EstFeuille) {
      for (int j = Droite.Longueur; j >= 1; j--)      // deplacements
        Droite.Liste_Fils [j] = Droite.Liste_Fils [j - 1]; 
      Droite.Liste_Fils [0] = Gauche.Liste_Fils[Degre_G]; 
    } 
    Gauche.Longueur = Gauche.Longueur - 1; 
  } 
} 
