//////////////////////////////////////////////////////////////
//                                                          //
//             MODULE DE STRUCTURES DE DONNEES              //
//                                                          //
//        MODULE ABSTRAIT DE RECHERCHE DANS UNE LISTE       //
//                                                          //
// Ce module assure les fonctions generales d'affichage et  //
//    de surveillance de l'execution d'une recherche        //
//    particuliere. Le controle se fait au moyen de boutons //
//    et d'un ascenceur. Il est possible d'arreter,         //
//    suspendre, reprendre ou executer en pas a pas la      //
//    recherche. L'ascenseur controle la vitesse            //
//    d'execution de la recherche. Lors de chaque           //
//    comparaison, les elements concernes sont affiches     //
//    dans une couleur particuliere. L'algorithme de        //
//    recherche n'est pas defini dans cette classe mais     //
//    dans les classes derivees.                            //
//                                                          //
// Auteur: Christian CARREZ,                                //
// Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   //
// Derniere modification: 17 fevrier 2000                   //
//////////////////////////////////////////////////////////////

package Demonstration_Recherche;

import java.awt.*; 
import java.awt.event.*; 
import Commun_Demonstration.* ;

class Affiche_Recherche extends Canvas { 

  private Algorithmes_Recherche Demo;
  private Image Image_Locale;

  public Affiche_Recherche (Algorithmes_Recherche Demo) { 
    this.Demo = Demo;
    Image_Locale = createImage (1000, 1000) ; 
  }

  public void paint (Graphics g) { 
    Rectangle r = getBounds() ;
    int Largeur;
    int Pas;
    int Hauteur;
    int cx, cy; 
    Hauteur = (r.height - 20) / Demo.ieme.length; 
    cy = 10 + (r.height - Hauteur * Demo.ieme.length) / 2;
    Pas = (r.width - 20) / (Demo.ieme.length + 2);
    Largeur = Pas / 2; 
    cx = (r.width - Pas * (Demo.ieme.length + 2)) / 2; 
    if (Demo.Valeur_U > 0) { 
      g.setColor (Color.green);
      g.fillRect (cx, 
                  cy + (Demo.ieme.length - Demo.Valeur_U) * Hauteur,
                  Largeur, Demo.Valeur_U * Hauteur);
    } 
    cx = cx + Pas;
    for (int i = 0; i < Demo.ieme.length; i++) { 
      cx = cx + Pas;
      if (Demo.ieme[i] > 0) { 
        g.setColor ((Color) Demo.Colors[i]);
        g.fillRect (cx, 
                   cy + (Demo.ieme.length - Demo.ieme[i]) * Hauteur,
                     Largeur, Demo.ieme[i] * Hauteur);
      } 
    } 
  } 

  public void update (Graphics g) { 
    Rectangle r = getBounds() ;
    if (Image_Locale == null) 
      Image_Locale = createImage(1000, 1000) ;
    Graphics GL = Image_Locale.getGraphics();
    GL.setColor (getBackground ());
    GL.fillRect (0, 0, r.width, r.height);
    paint (GL);
    g.drawImage (Image_Locale, 0, 0, this);
  }
} 

public abstract class Algorithmes_Recherche extends Thread { 
  private Affiche_Recherche afficheDemo; 
  protected ControleDemo controleDemo;
  private Frame f;
  int ieme [];
  Object Colors[];
  int Valeur_U;
  int Valeur_V; 
 
  public Algorithmes_Recherche (int[] V_Init,
                           int L_Anim, int H_Anim, String Title) { 
    f = new Frame (Title) ;
    ieme = new int [V_Init.length];
    Colors = new Color[ieme.length];
    for (int i = 0; i < ieme.length; i++) { 
      ieme[i] = V_Init[i];
      Colors[i] = Color.red;
    } 
    f.setLayout(new BorderLayout()) ;
    afficheDemo = new Affiche_Recherche (this); 
    f.add("Center", afficheDemo);
    controleDemo = new ControleDemo (false) ;
    f.add("South", controleDemo); 
    f.setSize (L_Anim, H_Anim); 
    f.show();
  } 

  protected void Redessiner () throws StopAnim { 
    afficheDemo.repaint ();
    controleDemo.Suspendre();
  } 

  abstract void La_Recherche () throws StopAnim; 

  public void run () { 
    try { 
      La_Recherche ();
    } catch (StopAnim e) { 
    }
    controleDemo.AttenteArret();
    f.dispose();
  }
} 
