//////////////////////////////////////////////////////////////
//                                                          //
//             MODULE DE STRUCTURES DE DONNEES              //
//                                                          //
//    MODULE D'ANIMATION DE LA GESTION D'UNE LISTE EN TAS   //
//                                                          //
// Ce module assure les fonctions generales d'affichage et  //
//    de surveillance de l'execution des operations sur un  //
//    tas, en visualisant l'arbre binaire ainsi que la      //
//    liste. Le controle se fait au moyen de boutons et     //
//    d'un ascenceur. Il est possible d'arreter, suspendre, //
//    reprendre ou executer en pas a pas les oprations.    //
//    L'ascenseur controle la vitesse d'execution d'une     //
//    opration. Lors de chaque comparaison, ou chaque      //
//    transfert, les elements concernes sont affiches dans  //
//    une couleur particuliere.                             //
// Note: la liste est la representation d'un arbre parfait  //
// preconditions                                            //
//    Ordonner_Le_Tas(La_Liste, Jusque):                    //
//        1 <= Jusque <= Longueur(La_Liste)                 //
//    Min_Du_Tas(La_Liste), Supprimer_Du_Tas(La_Liste):     //
//        Longueur(La-Liste) /= 0 et est-tas(La_Liste)      //
//    Ajuster_Le_Tas(La_Liste,En,Jusque):                   //
//        1 <= En <= Jusque <= Longueur(La_Liste)           //
//        et il existe v telle que                          //
//          est-tas(changer-ieme(La_Liste(1..Jusque),En,v)) //
//        i.e. cest un tas sauf eventuellement en En       //
// contrainte dimplantation                                //
//    Ajouter_Au_Tas(La_Liste, Val):                        //
//        Longueur (La_Liste) < La_Liste.Taille_Max         //
// proprietes fonctionnelles                                //
//    est-tas(La_Liste) =                                   //
//        pour tout 2<=i<=Longueur(La_Liste):               //
//            Ieme(La_Liste,i/2) <= Ieme(La_Liste,i)        //
//    Ordonner_Le_Tas(La_Liste, Jusque) = l1 & l2 ou`       //
//        est-permut(l1,La_Liste(1..Jusque)) et est-tas(l1) //
//        et l2 = La_Liste(Jusque+1..Longueur(La_Liste))    //
//    Ajouter_Au_Tas(La_Liste, Val) = l telle que:          //
//        est-permut(l, prolonger(La_Liste), Val))          //
//        et est-tas(l)                                     //
//    Supprimer_Du_Tas(La_Liste) = l telle que:             //
//        est-permut(l,supprimer(La_Liste,1)) et est-tas(l) //
//    Ajuster_Le_Tas(La_Liste, En, Jusque) = ll & l2 ou`    //
//        est-permut(l1,La_Liste(1..Jusque)) et est-tas(l1) //
//        et l2 = La_Liste(Jusque+1..Longueur(La_Liste))    //
//                                                          //
// Auteur: Christian CARREZ,                                //
// Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   //
// Derniere modification: 17 fevrier 2000                   //
//////////////////////////////////////////////////////////////

package Demonstration_Recherche;

import java.awt.*; 
import java.awt.event.*;
import Commun_Demonstration.* ;

class NatureOperation extends Panel implements ActionListener { 
  private TextField Texte_Valeur; 
  private TextField Texte_Position;
  private TextField Phase;
  int Valeur_Selectionnee;
  int Position; 
  final int RIEN = 0; 
  final int CHANGER = 1; 
  final int AJOUTER = 2; 
  final int SUPPRIMER = 3;
  final int FORCER = 4;
  int Operation = RIEN;

  public NatureOperation () { 
    Button b = null ; 
    Phase = new TextField ("Attente", 15);
    Phase.setEditable (false);
//    Phase.setFont (new Font ("middleFont", 0, Taille_Fonte));
    add(Phase); 
    add (new Label ("Valeur: "));
    Texte_Valeur = new TextField ("", 2); 
    add(Texte_Valeur); 
    add (new Label ("Position: "));
    Texte_Position = new TextField ("", 2); 
    add(Texte_Position);
    b = new Button ("Changer");
    add(b) ; 
    b.addActionListener (this) ;
    b = new Button ("Forcer");
    add(b) ; 
    b.addActionListener (this) ;
    b = new Button ("Ajouter");
    add(b) ; 
    b.addActionListener (this) ; 
    b = new Button ("Supprimer");
    add(b) ; 
    b.addActionListener (this) ; 
  }
 
  public void actionPerformed (ActionEvent evt) { 
    String Etiquette = evt.getActionCommand();
    if (Operation != RIEN) return;
    if (Etiquette.equals ("Changer")) {
      try { 
          Valeur_Selectionnee = 
                            Integer.parseInt (Texte_Valeur.getText());
          Position = Integer.parseInt (Texte_Position.getText());
          Operation = CHANGER; 
      } catch (Exception e) { } 
    } else if (Etiquette.equals ("Forcer")) { 
      try {
        Position = Integer.parseInt (Texte_Position.getText());
        Operation = FORCER; 
      } catch (Exception e) { }
    } else if (Etiquette.equals ("Ajouter")) { 
      try {
        Valeur_Selectionnee = 
                            Integer.parseInt (Texte_Valeur.getText());
        Operation = AJOUTER; 
      } catch (Exception e) { }
    } else if (Etiquette.equals ("Supprimer")) { 
        Operation = SUPPRIMER; 
    } 
  }

  public void Affiche_Phase (String S) { 
    Phase.setText (S);
  } 
} 

class Affiche_Le_Tas extends Canvas { 
  private Gestion_Du_Tas Demo ;
  private Image Image_Locale ; 
  private int Cx[];

  public Affiche_Le_Tas (Gestion_Du_Tas Demo) { 
    this.Demo = Demo ; 
    Image_Locale = createImage (1200, 1000);
    Cx = new int [Demo.ieme.length]; // positionnement horizontal
  } 

  public void paint (Graphics g) { 
    Rectangle r = getBounds() ; 
    int Pas_H = (r.width - 20) / 2; 
    int Hauteur = (2 * r.height) / 3 ; 
    int Val_Max = Demo.Valeur_U;
    int Largeur; 
    int cx;
    int Deb_Niveau = 2;               // debut du prochain niveau 
    int Pas_V;
    int cy; 
    int Taille_Fonte = r.width / 32 - 6; 
    int Demi_H_Label; // par rapport au centre
    int Demi_V_Label;
    int Demi_H_Noeud; // taille du noeud
    int Demi_V_Noeud; 
    if (Taille_Fonte < 8) Taille_Fonte = 8; 
    if (Taille_Fonte > 16) Taille_Fonte = 16; 
    Demi_V_Label = (Taille_Fonte / 2) - 1;
    Demi_H_Label = Demi_V_Label + 2;
    Demi_V_Noeud = Demi_H_Label;
    Demi_H_Noeud = Demi_V_Noeud + 1;
    Pas_V = (Hauteur - 2 * Demi_V_Noeud - 20) / Demo.H_Arbre;
    Hauteur = 20 + 2 * Demi_V_Noeud + Pas_V * Demo.H_Arbre;
    cy = 10 + Demi_V_Noeud; 
    Cx[0] = Pas_H;
    g.setFont (new Font("bigFont", 0, Taille_Fonte)); 
    if (Demo.Valeur_U != 0) {
      g.fillRect (10, cy - Demi_V_Noeud,
                  2 * Demi_H_Noeud, 2 * Demi_V_Noeud); 
      g.setColor (Color.black);
      g.drawString("" + Demo.Valeur_U, 11, cy + Demi_V_Label);
    }
    for (int i = 0; i < Demo.Dernier; i++) { 
      if (i > 0) { 
        Cx[i] = Cx[(i-1)/2] + (1 - (i%2) * 2) * Pas_H;
        g.setColor (Color.black); 
        g.drawLine (Cx[(i-1)/2] + (1 - (i%2) * 2) * Demi_H_Noeud, 
                    cy - Pas_V + Demi_V_Noeud,
                    Cx[i], cy - Demi_V_Noeud); 
      } 
      g.setColor ((Color) Demo.Colors[i]); 
      if (Demo.ieme[i] != 0) {
        g.fillRect (Cx[i] - Demi_H_Noeud, cy - Demi_V_Noeud,
                  2 * Demi_H_Noeud, 2 * Demi_V_Noeud); 
        g.setColor (Color.black);
        g.drawString("" + Demo.ieme[i], 
                     Cx[i] - Demi_H_Label, cy + Demi_V_Label);
      } else 
        g.drawRect (Cx[i] - Demi_H_Noeud, cy - Demi_V_Noeud,
                  2 * Demi_H_Noeud, 2 * Demi_V_Noeud);
      if (i + 2 == Deb_Niveau) { 
        cy = cy + Pas_V;
        Deb_Niveau = 2 * Deb_Niveau;
        Pas_H = Pas_H / 2; 
      } 
    } 
    for (int i = 0; i < Demo.ieme.length; i++) 
      if (Demo.ieme[i] > Val_Max) Val_Max = Demo.ieme[i];
    if (Val_Max == 0) return;
    Pas_V = (r.height - Hauteur - 10) / Val_Max; 
    if (Pas_V == 0) Pas_V = 1;
    cy = Hauteur + (r.height - Hauteur - Pas_V * Val_Max) / 2;
    Pas_H = (r.width - 20) / (Demo.ieme.length + 2);
    Largeur = Pas_H / 2; 
    cx = (r.width - Pas_H * (Demo.ieme.length + 2)) / 2; 
    if (Demo.Valeur_U > 0) { 
      g.setColor (Color.green);
      g.fillRect (cx, 
                  cy + (Val_Max - Demo.Valeur_U) * Pas_V,
                  Largeur, Demo.Valeur_U * Pas_V);
    } 
    cx = cx + Pas_H;
    g.setColor (Color.blue);
    g.drawLine (cx + Demo.Dernier * Pas_H + 3 * Pas_H / 4, cy,
                cx + Demo.Dernier * Pas_H + 3 * Pas_H / 4, 
                cy + Val_Max * Pas_V);
    for (int i = 0; i < Demo.ieme.length; i++) { 
      cx = cx + Pas_H;
      if (Demo.ieme[i] > 0) { 
        g.setColor ((Color) Demo.Colors[i]);
        g.fillRect (cx, 
                    cy + (Val_Max - Demo.ieme[i]) * Pas_V,
                    Largeur, Demo.ieme[i] * Pas_V);
      } 
      if ((i + 1) % 5 == 0) { 
        g.setColor (Color.black);
        g.drawString("" + (i + 1), 
                     cx, cy + Val_Max * Pas_V + 2 * Demi_V_Label);
      } 
    }
  } 
 
  public void update (Graphics g) { 
    Rectangle r = getBounds() ;
    if (Image_Locale == null) 
      Image_Locale = createImage(1200, 1000) ;
    Graphics GL = Image_Locale.getGraphics();
    GL.setColor (getBackground ());
    GL.fillRect (0, 0, r.width, r.height);
    paint (GL);
    g.drawImage (Image_Locale, 0, 0, this);
  }
} 
 
public class Gestion_Du_Tas extends Thread { 
  private NatureOperation natureOperation;
  private Affiche_Le_Tas afficheDemo; 
  private ControleDemo controleDemo;
  private Frame f;
  int ieme [];
  Object Colors[];
  int Valeur_U;
  int Valeur_V; 
  int Dernier;
  int H_Arbre;
 
  public Gestion_Du_Tas (int[] V_Init,
                           int L_Anim, int H_Anim) { 
    f = new Frame ("Animation des arbres d'une liste en tas");
    ieme = new int [V_Init.length];
    Colors = new Color[ieme.length];
    Dernier = V_Init.length;
    for (int i = 0; i < Dernier; i++) { 
      ieme[i] = 2 * V_Init[i];
      Colors[i] = Color.pink;
    } 
    H_Arbre = (int) (Math.log(ieme.length) / Math.log (2));
    f.setLayout(new BorderLayout());
    natureOperation = new NatureOperation ();
    f.add("North", natureOperation);
    afficheDemo = new Affiche_Le_Tas (this) ;
    f.add("Center", afficheDemo);
    controleDemo = new ControleDemo (false) ;
    f.add("South", controleDemo); 
    f.setSize (L_Anim, H_Anim); 
    f.show();
  } 

  protected void Redessiner () throws StopAnim { 
    afficheDemo.repaint ();
    controleDemo.Suspendre();
  } 

  private void Ordonner_Vers_Le_Bas (int En) throws StopAnim { 
    int Le_Fils;                        // indice du fils
    int Le_Pere = En;                    // indice du pre
    Valeur_U = ieme[En - 1]; ieme[En - 1] = 0; 
    Colors[En-1] = Color.magenta;
    Redessiner();
    while (2 * Le_Pere <= Dernier) {            // Il y a un fils
      Le_Fils = 2 * Le_Pere;    // Choisir le plus petit fils
      if (Le_Fils < Dernier) {                // il a un frre
        Colors[Le_Fils-1] = Color.green;
        Colors[Le_Fils] = Color.green;
        Redessiner();
        if (ieme[Le_Fils-1] > ieme[Le_Fils] ) { 
          Colors[Le_Fils-1] = Color.pink;
          Le_Fils = Le_Fils + 1;            // qui est plus petit
        } else Colors[Le_Fils] = Color.pink; 
      };
      Colors[Le_Fils-1] = Color.green;
      Redessiner();
      Colors[Le_Fils-1] = Color.pink;
      if (ieme[Le_Fils-1] > Valeur_U) break;        // bien plac
      ieme[Le_Pere-1] = ieme[Le_Fils-1]; ieme[Le_Fils-1] = 0;
      Redessiner();
      Le_Pere = Le_Fils; // remonter le fils et poursuivre dessous
    } 
    ieme[Le_Pere-1] = Valeur_U; Valeur_U = 0;       // place finale
    Redessiner();
    Colors[En-1] = Color.pink;
  } 

  private void Ordonner_Vers_Le_Haut (int En) throws StopAnim {
    int Le_Fils = En;                          // indice du fils
    int Le_Pere;                               // indice du pre
    Valeur_U = ieme[En-1]; ieme[En - 1] = 0; 
    Colors[En-1] = Color.magenta;
    Redessiner();
    while (Le_Fils != 1) {                        // le pere existe
      Le_Pere = Le_Fils / 2; 
      Colors[Le_Pere-1] = Color.green; ;
      Redessiner();
      Colors[Le_Pere-1] = Color.pink;
      if (ieme[Le_Pere-1] < Valeur_U) break; 
      ieme[Le_Fils-1] = ieme[Le_Pere-1]; ieme[Le_Pere-1] = 0; 
      Redessiner();
      Le_Fils = Le_Pere;                  // on remonte dans l'arbre
    }
    ieme[Le_Fils-1] = Valeur_U; Valeur_U = 0;         // place finale
    Redessiner();
    Colors[En-1] = Color.pink;
  } 

  private void Ordonner_Le_Tas (int Jusque) throws StopAnim { 
    int K;                                   // construction du tas
    Dernier = Jusque;
    for (K = Jusque/2; K >= 1; K--) { 
      Ordonner_Vers_Le_Bas (K); 
      controleDemo.FinEtape() ;
    }
  }

  public void Ajouter_Au_Tas (int Val) throws StopAnim { 
    if (Dernier < ieme.length) { 
      Dernier = Dernier + 1;
      ieme[Dernier - 1] = Val; 
      Ordonner_Vers_Le_Haut (Dernier); 
    }
  } 

  public void Supprimer_Du_Tas ()throws StopAnim { 
    if (Dernier != 0) { 
      ieme[0] = ieme[Dernier - 1];
      Dernier = Dernier - 1;
      if (Dernier > 1) Ordonner_Vers_Le_Bas (1); 
    }
  } 

  public void Ajuster_Le_Tas (int En) throws StopAnim {
    if (En <= Dernier) {
      if (2 * En <= Dernier) 
        Ordonner_Vers_Le_Bas (En);// ajustement vers les fils
      if (En > 1) 
        Ordonner_Vers_Le_Haut (En); // ajustement vers le pere
    }
  }

  void Le_Tas () throws StopAnim { 
    natureOperation.Affiche_Phase ("ordonner le tas"); 
    Ordonner_Le_Tas (Dernier); 
    natureOperation.Affiche_Phase ("Tas ordonne ");
    while (!controleDemo.DemandeArret()) { 
      if (natureOperation.Operation == natureOperation.RIEN) try { 
          sleep (100);
      } catch (Exception e) {
      } 
      if (!controleDemo.DemandeArret() 
           && natureOperation.Operation != natureOperation.RIEN) { 
        switch (natureOperation.Operation) { 
          case natureOperation.CHANGER: 
            natureOperation.Affiche_Phase ("Changement ");
           if (natureOperation.Position >= 1 
                 && natureOperation.Position <= ieme.length) { 
              ieme[natureOperation.Position - 1] = 
                            natureOperation.Valeur_Selectionnee;
              if (natureOperation.Position <= Dernier) 
                Ajuster_Le_Tas (natureOperation.Position); 
            } 
            break; 
          case natureOperation.FORCER: 
            natureOperation.Affiche_Phase ("Ordonner le tas ");
           if (natureOperation.Position >= 1 
                 && natureOperation.Position <= ieme.length) 
              Ordonner_Le_Tas (natureOperation.Position); 
            break;
          case natureOperation.AJOUTER: 
            natureOperation.Affiche_Phase ("Ajout de la valeur ");
            Ajouter_Au_Tas (natureOperation.Valeur_Selectionnee); 
            break; 
          case natureOperation.SUPPRIMER: 
            natureOperation.Affiche_Phase ("Supprimer minimum ");
            Supprimer_Du_Tas ();
            break; 
        } 
        natureOperation.Operation = natureOperation.RIEN;
        Redessiner (); 
        natureOperation.Affiche_Phase ("Operation terminee ");
      } 
    } 
  } 

  public void run () { 
    try { 
      Le_Tas ();
    } catch (StopAnim e) { 
    }
    controleDemo.AttenteArret();
    f.dispose();
  }
} 
