//////////////////////////////////////////////////////////////
//                                                          //
//             MODULE DE STRUCTURES DE DONNEES              //
//                                                          //
//          MODULE ABSTRAIT DES ALGORITHMES DE TRI          //
//                                                          //
// Ce module assure les fonctions generales d'affichage et  //
//    de surveillance de l'execution d'un tri particulier.  //
//    Le controle se fait au moyen de boutons et d'un       //
//    ascenceur. Il est possible d'arreter, suspendre,      //
//    reprendre ou executer en pas a pas le tri. L'ascenseur//
//    controle la vitesse d'execution du tri.               //
//    Lors de chaque comparaison, ou chaque transfert, les  //
//    elements concernes sont affiches dans une couleur     //
//    particuliere.                                         //
//    L'algorithme de tri n'est pas defini dans cette classe//
//    mais dans les classes derivees.                       //
//                                                          //
// Auteur: Christian CARREZ,                                //
// Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   //
// Derniere modification: 17 fevrier 2000                   //
//////////////////////////////////////////////////////////////

package Demonstration_Tri;

import java.awt.*; 
import java.awt.event.*;
import Commun_Demonstration.*;

class Affiche_Tri extends Canvas { 
  private Algorithmes_Tri Demo ;
  private Image Image_Locale ;

  public Affiche_Tri (Algorithmes_Tri Demo) { 
    this.Demo = Demo ;
    Image_Locale = createImage (1000, 1000);
  } 

  public void paint (Graphics g) { 
    Rectangle r = getBounds() ;
    int Largeur;
    int Pas;
    int Hauteur;
    int cx, cy; 
    boolean Trait_Bleu = false;
    int H_Bleu = 0; 
    Hauteur = (r.height - 20) / Demo.ieme.length; 
    cy = 10 + (r.height - Hauteur * Demo.ieme.length) / 2;
    Pas = (r.width - 20) / (Demo.ieme.length + 3);
    Largeur = Pas / 2; 
    cx = (r.width - Pas * (Demo.ieme.length + 3)) / 2;
    if (Demo.Valeur_V > 0) { 
      H_Bleu = cy + (Demo.ieme.length - Demo.Valeur_V) * Hauteur;
      g.setColor (Color.blue);
      g.fillRect (cx, H_Bleu, Largeur, Demo.Valeur_V * Hauteur);
    }
    cx = cx + Pas;
    if (Demo.Valeur_U > 0) { 
      g.setColor (Color.green);
      g.fillRect (cx, 
                  cy + (Demo.ieme.length - Demo.Valeur_U) * Hauteur,
                  Largeur, Demo.Valeur_U * Hauteur);
    } 
    cx = cx + Pas;
    for (int i = 0; i < Demo.ieme.length; i++) { 
      cx = cx + Pas;
      if (Demo.ieme[i] > 0) { 
        g.setColor ((Color) Demo.Colors[i]);
        g.fillRect (cx, 
                    cy + (Demo.ieme.length - Demo.ieme[i]) * Hauteur,
                    Largeur, Demo.ieme[i] * Hauteur);
      } 
      if (H_Bleu > 0) { 
        if (Trait_Bleu) 
          Trait_Bleu = !Color.blue.equals (Demo.Colors[i]); 
        else 
         Trait_Bleu = Color.blue.equals (Demo.Colors[i]); 
        if (Trait_Bleu) {
          g.setColor (Color.blue);
          g.drawLine (cx, H_Bleu, cx + Pas, H_Bleu);
        }
      } 
    } 
  } 
 
  public void update (Graphics g) { 
    Rectangle r = getBounds() ;
    if (Image_Locale == null) 
      Image_Locale = createImage(1000, 1000) ;
    Graphics GL = Image_Locale.getGraphics();
    GL.setColor (getBackground ());
    GL.fillRect (0, 0, r.width, r.height);
    paint (GL);
    g.drawImage (Image_Locale, 0, 0, this);
  }
} 
 
public abstract class Algorithmes_Tri extends Thread { 
  private Affiche_Tri afficheDemo; 
  protected ControleDemo controleDemo;
  int ieme [];
  Object Colors[];
  int Valeur_U;
  int Valeur_V; 
  Frame f;
 
  public Algorithmes_Tri (int[] V_Init,
                           int L_Anim, int H_Anim, String Title) { 
    f = new Frame (Title) ;
    ieme = new int [V_Init.length];
    Colors = new Color[ieme.length];
    for (int i = 0; i < ieme.length; i++) { 
      ieme[i] = V_Init[i];
      Colors[i] = Color.red;
    } 
    f.setLayout(new BorderLayout());
    afficheDemo = new Affiche_Tri (this) ;
    f.add("Center", afficheDemo);
    controleDemo = new ControleDemo (false) ;
    f.add("South", controleDemo); 
    f.setSize (L_Anim, H_Anim); 
    f.show();
  } 

  protected void Redessiner () throws StopAnim { 
    afficheDemo.repaint ();
    controleDemo.Suspendre();
  } 

  protected void Echanger (int I, int J) throws StopAnim { 
    Valeur_U = ieme [J]; ieme[J] = 0; Redessiner ();
    ieme [J] = ieme [I]; ieme[I] = 0; Redessiner ();
    ieme [I] = Valeur_U; Valeur_U = 0; Redessiner ();
  }

  abstract void Le_Tri () throws StopAnim; 

  public void run () { 
    try { 
      Le_Tri ();
    } catch (StopAnim e) { 
    } 
    controleDemo.AttenteArret();
    f.dispose();
  } 
} 
