//////////////////////////////////////////////////////////////
//                                                          //
//             MODULE DE STRUCTURES DE DONNEES              //
//                                                          //
//       MODULE D'ANIMATION DE LA DEMO DU TRI PAR TAS       //
//                                                          //
// Ce module assure les fonctions generales d'affichage et  //
//    de surveillance de l'execution du tri par tas, en     //
//    visualisant l'arbre binaire ainsi que la liste.       //
//    Le controle se fait au moyen de boutons et d'un       //
//    ascenceur. Il est possible d'arreter, suspendre,      //
//    reprendre ou executer en pas a pas le tri. L'ascenseur//
//    controle la vitesse d'execution du tri.               //
//    Lors de chaque comparaison, ou chaque transfert, les  //
//    elements concernes sont affiches dans une couleur     //
//    particuliere.                                         //
// Note: la liste est la representation d'un arbre parfait  //
//    premiere etape: on le rend partiellement ordonne      //
//    deuxieme etape: on extrait les elements en conservant //
//        la propriete partiellement ordonne                //
// preconditions                                            //
//    ordonner (a):                                         //
//    a /= vide implique (g(a) = vide implique d(a) = vide) //
// proprietes fonctionnelles                                //
//    ordonner(a) =                                         //
//        si a = vide ou a = <o, vide, vide> alors a        //
//        sinon soit a = <o, <o1, gg, dd>, d>;              //
//            si o>=o1 et (d=vide ou o>=racine(d)) alors a  //
//            sinsi o<o1 et (d=vide ou o1>=racine(d)) alors //
//                <o1, ordonner(<o, gg, dd>), d>            //
//            sinon soit a = <o, g, <o2, dg, dd>>;          //
//                <o2, g, ordonner(<o, dg, dd>)>            //
//    tri-sel-iter(l1, l) =                                 //
//        si l = listevide alors l1                         //
//        sinon tri-sel-iter(max(l)::l1, supprimer-max(l))  //
//        fsi                                               //
//    tri (l) = tri-sel-iter(listevide, l)                  //
//                                                          //
// Auteur: Christian CARREZ,                                //
// Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   //
// Derniere modification: 17 fevrier 2000                   //
//////////////////////////////////////////////////////////////

package Demonstration_Tri;

import java.awt.*; 
import java.awt.event.*;
import Commun_Demonstration.* ;

class Affiche_Tas extends Canvas { 
  private Animation_Tas Demo ;
  private Image Image_Locale ; 
  private int Cx[];

  public Affiche_Tas (Animation_Tas Demo) { 
    this.Demo = Demo ; 
    Image_Locale = createImage (1200, 1000);
    Cx = new int [Demo.ieme.length]; // positionnement horizontal
  } 

  public void paint (Graphics g) { 
    Rectangle r = getBounds() ; 
    int Pas_H = (r.width - 20) / 2; 
    int Hauteur = (2 * r.height) / 3 ; 
    int Largeur; 
    int cx;
    int Deb_Niveau = 2;               // debut du prochain niveau 
    int Pas_V;
    int cy; 
    int Taille_Fonte = r.width / 32 - 6; 
    int Demi_H_Label; // par rapport au centre
    int Demi_V_Label;
    int Demi_H_Noeud; // taille du noeud
    int Demi_V_Noeud; 
    if (Taille_Fonte < 8) Taille_Fonte = 8; 
    if (Taille_Fonte > 16) Taille_Fonte = 16; 
    Demi_V_Label = (Taille_Fonte / 2) - 1;
    Demi_H_Label = Demi_V_Label + 2;
    Demi_V_Noeud = Demi_H_Label;
    Demi_H_Noeud = Demi_V_Noeud + 1;
    Pas_V = (Hauteur - 2 * Demi_V_Noeud - 20) / Demo.H_Arbre;
    Hauteur = 20 + 2 * Demi_V_Noeud + Pas_V * Demo.H_Arbre;
    cy = 10 + Demi_V_Noeud; 
    Cx[0] = Pas_H;
    g.setFont (new Font("bigFont", 0, Taille_Fonte)); 
    if (Demo.Valeur_U != 0) {
      g.fillRect (10, cy - Demi_V_Noeud,
                  2 * Demi_H_Noeud, 2 * Demi_V_Noeud); 
      g.setColor (Color.black);
      g.drawString("" + Demo.Valeur_U, 11, cy + Demi_V_Label);
    }
    for (int i = 0; i < Demo.Dernier; i++) { 
      if (i > 0) { 
        Cx[i] = Cx[(i-1)/2] + (1 - (i%2) * 2) * Pas_H;
        g.setColor (Color.black); 
        g.drawLine (Cx[(i-1)/2] + (1 - (i%2) * 2) * Demi_H_Noeud, 
                    cy - Pas_V + Demi_V_Noeud,
                    Cx[i], cy - Demi_V_Noeud); 
      } 
      g.setColor ((Color) Demo.Colors[i]); 
      if (Demo.ieme[i] != 0) {
        g.fillRect (Cx[i] - Demi_H_Noeud, cy - Demi_V_Noeud,
                  2 * Demi_H_Noeud, 2 * Demi_V_Noeud); 
        g.setColor (Color.black);
        g.drawString("" + Demo.ieme[i], 
                     Cx[i] - Demi_H_Label, cy + Demi_V_Label);
      } else 
        g.drawRect (Cx[i] - Demi_H_Noeud, cy - Demi_V_Noeud,
                  2 * Demi_H_Noeud, 2 * Demi_V_Noeud);
      if (i + 2 == Deb_Niveau) { 
        cy = cy + Pas_V;
        Deb_Niveau = 2 * Deb_Niveau;
        Pas_H = Pas_H / 2; 
      } 
    } 
    Pas_V = (r.height - Hauteur - 10) / Demo.ieme.length; 
    cy = Hauteur 
         + (r.height - Hauteur - Pas_V * Demo.ieme.length) / 2;
    Pas_H = (r.width - 20) / (Demo.ieme.length + 2);
    Largeur = Pas_H / 2; 
    cx = (r.width - Pas_H * (Demo.ieme.length + 2)) / 2; 
    if (Demo.Valeur_U > 0) { 
      g.setColor (Color.green);
      g.fillRect (cx, 
                  cy + (Demo.ieme.length - Demo.Valeur_U) * Pas_V,
                  Largeur, Demo.Valeur_U * Pas_V);
    } 
    cx = cx + Pas_H;
    for (int i = 0; i < Demo.ieme.length; i++) { 
      cx = cx + Pas_H;
      if (Demo.ieme[i] > 0) { 
        g.setColor ((Color) Demo.Colors[i]);
        g.fillRect (cx, 
                    cy + (Demo.ieme.length - Demo.ieme[i]) * Pas_V,
                    Largeur, Demo.ieme[i] * Pas_V);
      } 
    }
  } 
 
  public void update (Graphics g) { 
    Rectangle r = getBounds() ;
    if (Image_Locale == null) 
      Image_Locale = createImage(1200, 1000) ;
    Graphics GL = Image_Locale.getGraphics();
    GL.setColor (getBackground ());
    GL.fillRect (0, 0, r.width, r.height);
    paint (GL);
    g.drawImage (Image_Locale, 0, 0, this);
  }
} 
 
public class Animation_Tas extends Thread { 
  private Affiche_Tas afficheDemo; 
  private ControleDemo controleDemo;
  private Frame f;
  int ieme [];
  Object Colors[];
  int Valeur_U;
  int Valeur_V; 
  int Dernier;
  int H_Arbre;
 
  public Animation_Tas (int[] V_Init,
                           int L_Anim, int H_Anim) { 
    f = new Frame ("Animation des arbres du tri par tas");
    ieme = new int [V_Init.length];
    Colors = new Color[ieme.length];
    for (int i = 0; i < ieme.length; i++) { 
      ieme[i] = V_Init[i];
      Colors[i] = Color.pink;
    } 
    H_Arbre = (int) (Math.log(ieme.length) / Math.log (2));
    f.setLayout(new BorderLayout());
    afficheDemo = new Affiche_Tas (this) ;
    f.add("Center", afficheDemo);
    controleDemo = new ControleDemo (false) ;
    f.add("South", controleDemo); 
    f.setSize (L_Anim, H_Anim); 
    f.show();
  } 

  protected void Redessiner () throws StopAnim { 
    afficheDemo.repaint ();
    controleDemo.Suspendre();
  } 

  private void Ordonner (int En, int Max) throws StopAnim { 
                           // Ordonner la liste en tas
    int Le_Fils;                        // indice du fils
    int Le_Pere = En;                    // indice du pre
    while (2 * Le_Pere <= Max) {            // Il y a un fils
      Le_Fils = 2 * Le_Pere;    // Choisir le plus grand fils
      if (Le_Fils < Max) {                // il a un frre
        Colors[Le_Fils-1] = Color.green;
        Colors[Le_Fils] = Color.green;
        Redessiner();
        if (ieme[Le_Fils-1] < ieme[Le_Fils] ) { 
          Colors[Le_Fils-1] = Color.pink;
          Le_Fils = Le_Fils + 1;            // qui est plus grand
        } else Colors[Le_Fils] = Color.pink; 
      };
      Colors[Le_Fils-1] = Color.green;
      Redessiner();
      Colors[Le_Fils-1] = Color.pink;
      if (ieme[Le_Fils-1] < Valeur_U) break;        // bien plac
      ieme[Le_Pere-1] = ieme[Le_Fils-1]; ieme[Le_Fils-1] = 0;
      Redessiner();
      Le_Pere = Le_Fils; // remonter le fils et poursuivre dessous
    } 
    ieme[Le_Pere-1] = Valeur_U; Valeur_U = 0;       // place finale
    Redessiner();
  } 

  void Le_Tri () throws StopAnim {  // tri par tas proprement dit
    int K;                                   // construction du tas
    Dernier = ieme.length;
    for (K = ieme.length/2; K >= 1; K--) {
      Valeur_U = ieme[K-1]; ieme[K-1] = 0; 
      Colors[K-1] = Color.magenta;
      Redessiner();
      Ordonner (K, ieme.length); 
      Colors[K-1] = Color.pink; 
      controleDemo.FinEtape() ;
    } 
                                         // extraction du tas
    for (K = ieme.length; K >= 2; K--) {
      Valeur_U = ieme[K-1]; ieme[K-1] = 0;      // dernire feuille
      Redessiner();
      ieme[K-1] = ieme[0]; ieme[0] = 0;
      Colors[K-1] = Color.magenta;
      Dernier = K - 1;
      Redessiner();              // le plus grand dans le rsultat
      Ordonner (1, K - 1);
              // ordonner avec la dernire feuille  la racine
      Colors[K-1] = Color.pink; 
      controleDemo.FinEtape() ;
    } 
    Dernier = -1;
    Redessiner();
  } 

  public void run () { 
    try { 
      Le_Tri ();
    } catch (StopAnim e) { 
    }
    controleDemo.AttenteArret();
    f.dispose();
  }
} 
