//////////////////////////////////////////////////////////////
//                                                          //
//             MODULE DE STRUCTURES DE DONNEES              //
//                                                          //
//                ALGORITHMES DE TRI RAPIDE                 //
//                                                          //
// Cette classe derive la la classe abstraite               //
//    Algorithmes_Tri et implemente le tri rapide           //
// preconditions                                            //
//    plus-petits(l), plus-grands(l), pivot(l):             //
//        non estvide(l)                                    //
// propriete fonctionnelle                                  //
//    tri-rapide(l) =                                       //
//        si l = listevide alors listevide                  //
//        sinon tri-rapide(plus-petits(l)) & [pivot(l)] &   //
//                           tri-rapide(plus-grands(l)) fsi //
//                                                          //
// Auteur: Christian CARREZ,                                //
// Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   //
// Derniere modification: 17 fevrier 2000                   //
//////////////////////////////////////////////////////////////

package Demonstration_Tri;

import java.awt.*; 
import Commun_Demonstration.*;

class Tri_Rapide extends Algorithmes_Tri { 
  private int[] Memo;
  private int Sommet = 0;
  public Tri_Rapide (int[] V_Init, int L_Anim, int H_Anim) { 
    super(V_Init, L_Anim, H_Anim, " tri rapide ");
    Memo = new int[20];
  } 

  void Le_Tri () throws StopAnim {            // tri rapide
    int Debut = 0; //dlimite la liste en cours de traitement
    int Fin = ieme.length-1;
    int Presumee;           // place o est la valeur sparante
    int I, J;            // pour rpartir
                       // le Presumee est dans Valeur_V
    if (ieme.length < 2 ) return;                  // rien  faire
    for (;;) { 
      while (Fin - Debut > 1) {    // Repartition 
        // des lments entre Debut et Fin compris, par rapport 
        //  une valeur mdiane. Suppose Fin - Debut >= 2
        Presumee = (Debut + Fin) / 2;
                      // choix parmis premier, milieu, dernier
        I = Debut;
        Colors[Presumee] = Color.green;
        Colors[I] = Color.green; ;
        Colors[Fin] = Color.blue;
        Redessiner();
        if (ieme[Presumee] < ieme[I] ) { 
          Colors[I] = Color.blue;
          I = Presumee; 
        } else Colors[Presumee] = Color.blue;
        Colors[Fin] = Color.green;
        Redessiner();
        if (ieme[Fin] < ieme[I] ) { 
          Colors[I] = Color.blue;
         I = Fin; 
        } else Colors[Fin] = Color.blue; 
        Colors[Debut] = Color.green;
        if (Debut != I) Echanger(Debut, I); // plus petit au dbut
        Colors[Debut] = Color.blue;
        Colors[Presumee] = Color.green;
        Colors[Fin] = Color.green;
        Redessiner();
        Colors[Fin] = Color.blue; 
        if (ieme[Presumee] < ieme[Fin] ) { 
          Valeur_V = ieme[Presumee]; ieme[Presumee] = 0;
          Redessiner();    // valeur mdiane est celle du milieu
        } else {
          Valeur_V = ieme[Fin]; ieme[Fin] = 0;  
          Redessiner();        // valeur mdiane est la dernire
          ieme[Fin] = ieme[Presumee]; ieme[Presumee] = 0;
          Redessiner();
        }
        Colors[Presumee] = Color.red;

         // La_Liste.Ieme (Debut) <= Valeur_V <= La_Liste.Ieme (Fin)
         // Valeur_V contient la valeur Pivot, 
        I = Debut + 1;               // placement des autres valeurs
        J = Fin - 1;
        while (I < J) {
          // Presumee repre une place vacante, 
          // de plus, I <= Presumee <= J
          while (I < Presumee) { 
            Colors[I] = Color.green;
            Redessiner();
            Colors[I] = Color.red;
            if (Valeur_V < ieme[I]) break; 
            I = I + 1; // OK
          } 
          while (J>Presumee) { 
            Colors[J] = Color.green;
            Redessiner();
            Colors[J] = Color.red;
            if (ieme[J] < Valeur_V) break;
            J = J - 1; // OK
          } 
          if (I == J) break;          // place de la valeur mdiane
          if (I == Presumee) {                     // cas 3
            ieme[I] = ieme[J]; ieme[J] = 0;             // dplacer
            Redessiner();
            Presumee = J;                  // nouvelle place vacante
            I = I + 1;
          } else if (J == Presumee) {             // cas 2
            ieme[J] = ieme[I]; ieme[I] = 0;           // dplacer
            Redessiner();
            Presumee = I;                  // nouvelle place vacante
            J = J - 1;
          } else {                             // cas 1
            Echanger (I, J);                 // dplacer les deux
            I = I + 1;
            J = J - 1;
          } 
        } 
        ieme[Presumee] = Valeur_V; Valeur_V = 0; // ranger le pivot
        Redessiner();
        controleDemo.FinEtape() ;
        Colors[Fin] = Color.red; 
        Colors[Debut] = Color.red;
                            // on a maintenant deux sous-listes
        if (Presumee - Debut < Fin - Presumee) {// plus petite d'abord
          Memo[Sommet] = Presumee+1; Sommet++;
          Memo[Sommet] = Fin; Sommet++;
          Fin = Presumee - 1;              // c'est celle du dbut
        } else { 
          Memo[Sommet] = Debut; Sommet++;
          Memo[Sommet] = Presumee - 1; Sommet++;
          Debut = Presumee + 1;           // c'est celle de la fin
        } 
      } 
                           // la liste est suffisamment petite
      if (Fin == Debut + 1) { 
        Colors[Fin] = Color.green; 
        Colors[Debut] = Color.green;
        Redessiner();
        if (ieme[Fin] < ieme[Debut] ) 
          Echanger (Debut, Fin); 
        Colors[Fin] = Color.red; 
        Colors[Debut] = Color.red; 
        controleDemo.FinEtape() ;
      } 
      if ( Sommet == 0 ) { 
        Redessiner();
        return;  // toutes les listes sont faites
      } 
      Sommet--; Fin = Memo[Sommet];
      Sommet--; Debut = Memo[Sommet]; 
    } 
  } 
} 
